import typer
from typer.testing import CliRunner
from commands.cli import app
import pytest
import tempfile
import os

runner = CliRunner()

def test_diff_basic():
    """Тест базовой функциональности diff"""
    config1_content = """
upstream backend {
    server 127.0.0.1:8080;
}
"""
    config2_content = """
upstream backend {
    server 127.0.0.1:8080;
    server 127.0.0.1:8081;
}
"""
    with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.conf') as f1, \
         tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.conf') as f2:
        f1.write(config1_content)
        f2.write(config2_content)
        f1.flush()
        f2.flush()
        config1_path = f1.name
        config2_path = f2.name
    
    try:
        result = runner.invoke(app, ["diff", config1_path, config2_path])
        assert result.exit_code == 0
        # Проверяем что diff показывает различия
    finally:
        os.unlink(config1_path)
        os.unlink(config2_path)

def test_diff_identical():
    """Тест diff для идентичных конфигов"""
    config_content = """
upstream backend {
    server 127.0.0.1:8080;
}
"""
    with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.conf') as f1, \
         tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.conf') as f2:
        f1.write(config_content)
        f2.write(config_content)
        f1.flush()
        f2.flush()
        config1_path = f1.name
        config2_path = f2.name
    
    try:
        result = runner.invoke(app, ["diff", config1_path, config2_path])
        assert result.exit_code == 0
    finally:
        os.unlink(config1_path)
        os.unlink(config2_path)

def test_diff_file_not_found():
    """Тест обработки отсутствующих файлов"""
    result = runner.invoke(app, ["diff", "/nonexistent1.conf", "/nonexistent2.conf"])
    # Команда должна вернуть exit code 1 при отсутствии файлов
    assert result.exit_code == 1




"""
Интеграционные тесты для полных сценариев использования nginx-lens
"""
import pytest
import tempfile
import os
from typer.testing import CliRunner
from commands.cli import app

runner = CliRunner()

def test_full_workflow_health_and_resolve():
    """Интеграционный тест: health + resolve для реального конфига"""
    config_content = """
events {
    worker_connections 1024;
}

http {
    upstream backend {
        server example.com:80;
        server 127.0.0.1:8080;
    }
    
    server {
        listen 80;
        location / {
            proxy_pass http://backend;
        }
    }
}
"""
    with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.conf') as f:
        f.write(config_content)
        f.flush()
        config_path = f.name
    
    try:
        # Тест resolve
        result_resolve = runner.invoke(app, ["resolve", config_path, "--max-workers", "5"])
        assert result_resolve.exit_code == 0
        assert "backend" in result_resolve.output
        
        # Тест health
        result_health = runner.invoke(app, ["health", config_path, "--timeout", "1", "--max-workers", "5"])
        assert result_health.exit_code in [0, 1]  # Может быть 0 или 1 в зависимости от доступности
        
        # Тест health с resolve
        result_health_resolve = runner.invoke(app, ["health", config_path, "--resolve", "--timeout", "1", "--max-workers", "5"])
        assert result_health_resolve.exit_code in [0, 1]
    finally:
        os.unlink(config_path)

def test_full_workflow_analyze_and_tree():
    """Интеграционный тест: analyze + tree для реального конфига"""
    config_content = """
events {
    worker_connections 1024;
}

http {
    upstream backend {
        server 127.0.0.1:8080;
    }
    
    server {
        listen 80;
        server_name example.com;
        
        location / {
            proxy_pass http://backend;
        }
        
        location /api {
            proxy_pass http://backend;
        }
    }
}
"""
    with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.conf') as f:
        f.write(config_content)
        f.flush()
        config_path = f.name
    
    try:
        # Тест analyze
        result_analyze = runner.invoke(app, ["analyze", config_path])
        assert result_analyze.exit_code == 0
        
        # Тест tree
        result_tree = runner.invoke(app, ["tree", config_path])
        assert result_tree.exit_code == 0
        assert "server" in result_tree.output.lower() or "location" in result_tree.output.lower()
    finally:
        os.unlink(config_path)

def test_full_workflow_diff_and_syntax():
    """Интеграционный тест: diff + syntax для реальных конфигов"""
    config1_content = """
events {
    worker_connections 1024;
}

http {
    upstream backend {
        server 127.0.0.1:8080;
    }
}
"""
    config2_content = """
events {
    worker_connections 2048;
}

http {
    upstream backend {
        server 127.0.0.1:8080;
        server 127.0.0.1:8081;
    }
}
"""
    with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.conf') as f1, \
         tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.conf') as f2:
        f1.write(config1_content)
        f2.write(config2_content)
        f1.flush()
        f2.flush()
        config1_path = f1.name
        config2_path = f2.name
    
    try:
        # Тест diff
        result_diff = runner.invoke(app, ["diff", config1_path, config2_path])
        assert result_diff.exit_code == 0
        
        # Тест syntax (может требовать nginx бинарь, поэтому проверяем только что команда работает)
        # Если nginx не установлен, команда вернет ошибку, но это нормально для теста
        result_syntax = runner.invoke(app, ["syntax", "--config", config1_path])
        # Exit code может быть любым, главное что команда выполнилась
        assert result_syntax.exit_code is not None
    finally:
        os.unlink(config1_path)
        os.unlink(config2_path)

def test_error_handling_workflow():
    """Интеграционный тест обработки ошибок"""
    # Тест с несуществующим файлом
    result = runner.invoke(app, ["health", "/nonexistent/nginx.conf"])
    assert result.exit_code == 1
    
    result = runner.invoke(app, ["resolve", "/nonexistent/nginx.conf"])
    assert result.exit_code == 1
    
    result = runner.invoke(app, ["analyze", "/nonexistent/nginx.conf"])
    assert result.exit_code != 0
    
    result = runner.invoke(app, ["tree", "/nonexistent/nginx.conf"])
    assert result.exit_code != 0
    
    result = runner.invoke(app, ["diff", "/nonexistent1.conf", "/nonexistent2.conf"])
    assert result.exit_code == 1
    
    result = runner.invoke(app, ["logs", "/nonexistent/log.log"])
    assert result.exit_code == 1

def test_exit_codes_workflow():
    """Интеграционный тест exit codes для CI/CD"""
    config_healthy = """
upstream backend {
    server 127.0.0.1:99999;  # Недоступный, но не критично для теста
}
"""
    config_unhealthy = """
upstream backend {
    server nonexistent-domain-12345.invalid:80;
}
"""
    with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.conf') as f1, \
         tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.conf') as f2:
        f1.write(config_healthy)
        f2.write(config_unhealthy)
        f1.flush()
        f2.flush()
        config1_path = f1.name
        config2_path = f2.name
    
    try:
        # Resolve с валидным DNS должен вернуть 0 или 1 (зависит от доступности)
        result1 = runner.invoke(app, ["resolve", config1_path])
        assert result1.exit_code in [0, 1]
        
        # Resolve с невалидным DNS должен вернуть 1
        result2 = runner.invoke(app, ["resolve", config2_path])
        # Может быть 0 если DNS не резолвится, но это нормально
        assert result2.exit_code is not None
    finally:
        os.unlink(config1_path)
        os.unlink(config2_path)




import typer
from rich.console import Console
from commands.health import health
from commands.analyze import analyze
from commands.tree import tree
from commands.diff import diff
from commands.route import route
from commands.include import include_tree
from commands.graph import graph
from commands.logs import logs
from commands.syntax import syntax
from commands.resolve import resolve
from commands.validate import validate
from commands.metrics import metrics
from commands.completion import app as completion_app
from commands.init import init

app = typer.Typer(help="nginx-lens — анализ и диагностика конфигураций Nginx")
console = Console()

app.command()(health)
app.command()(analyze)
app.command()(tree)
app.command()(diff)
app.command()(route)
app.command()(include_tree)
app.command()(graph)
app.command()(logs)
app.command()(syntax)
app.command()(resolve)
app.command()(validate)
app.command()(metrics)
app.command()(init)
app.add_typer(completion_app, name="completion", help="Генерация скриптов автодополнения")

if __name__ == "__main__":
    app() 


import typer
from typer.testing import CliRunner
from commands.cli import app
import pytest

runner = CliRunner()

def test_resolve_basic(monkeypatch):
    """Тест базовой функциональности resolve"""
    from commands import resolve as resolve_mod
    monkeypatch.setattr(resolve_mod, "parse_nginx_config", lambda path: type("T", (), {"get_upstreams": lambda self: {"test_up": ["example.com:80", "127.0.0.1:8080"]}})())
    monkeypatch.setattr(resolve_mod, "resolve_upstreams", lambda ups, max_workers: {
        "test_up": [
            {"address": "example.com:80", "resolved": ["93.184.216.34:80"]},
            {"address": "127.0.0.1:8080", "resolved": ["127.0.0.1:8080"]}
        ]
    })
    result = runner.invoke(app, ["resolve", "nginx.conf"])
    assert result.exit_code == 0
    assert "test_up" in result.output
    assert "example.com:80" in result.output
    assert "127.0.0.1:8080" in result.output

def test_resolve_failed(monkeypatch):
    """Тест resolve с невалидным DNS"""
    from commands import resolve as resolve_mod
    monkeypatch.setattr(resolve_mod, "parse_nginx_config", lambda path: type("T", (), {"get_upstreams": lambda self: {"test_up": ["nonexistent-domain-12345.invalid:80"]}})())
    monkeypatch.setattr(resolve_mod, "resolve_upstreams", lambda ups, max_workers: {
        "test_up": [
            {"address": "nonexistent-domain-12345.invalid:80", "resolved": []}
        ]
    })
    result = runner.invoke(app, ["resolve", "nginx.conf"])
    assert result.exit_code == 1
    assert "Failed to resolve" in result.output

def test_resolve_invalid_cname(monkeypatch):
    """Тест resolve с невалидным CNAME"""
    from commands import resolve as resolve_mod
    monkeypatch.setattr(resolve_mod, "parse_nginx_config", lambda path: type("T", (), {"get_upstreams": lambda self: {"test_up": ["example.com:80"]}})())
    monkeypatch.setattr(resolve_mod, "resolve_upstreams", lambda ups, max_workers: {
        "test_up": [
            {"address": "example.com:80", "resolved": ["invalid resolve (via cname.example.com -> TXT)"]}
        ]
    })
    result = runner.invoke(app, ["resolve", "nginx.conf"])
    assert result.exit_code == 1
    assert "invalid resolve" in result.output

def test_resolve_no_upstreams(monkeypatch):
    """Тест resolve когда нет upstream"""
    from commands import resolve as resolve_mod
    monkeypatch.setattr(resolve_mod, "parse_nginx_config", lambda path: type("T", (), {"get_upstreams": lambda self: {}})())
    result = runner.invoke(app, ["resolve", "nginx.conf"])
    assert result.exit_code == 0
    assert "не найдено" in result.output.lower() or "not found" in result.output.lower()

def test_resolve_max_workers_option(monkeypatch):
    """Тест опции --max-workers"""
    from commands import resolve as resolve_mod
    monkeypatch.setattr(resolve_mod, "parse_nginx_config", lambda path: type("T", (), {"get_upstreams": lambda self: {"test_up": ["example.com:80"]}})())
    monkeypatch.setattr(resolve_mod, "resolve_upstreams", lambda ups, max_workers: {
        "test_up": [{"address": "example.com:80", "resolved": ["93.184.216.34:80"]}]
    })
    result = runner.invoke(app, ["resolve", "nginx.conf", "--max-workers", "20"])
    assert result.exit_code == 0




import pytest
import time
from upstream_checker.checker import check_upstreams, resolve_upstreams

def test_parallel_processing_performance():
    """Тест производительности параллельной обработки"""
    # Создаем большое количество upstream для теста
    upstreams = {}
    for i in range(30):
        upstreams[f"backend{i}"] = [
            "127.0.0.1:99999",  # Недоступный порт для быстрого таймаута
            "example.com:80",
            "google.com:80"
        ]
    
    # Тест с max_workers=1 (последовательно)
    start_time = time.time()
    results_sequential = check_upstreams(upstreams, timeout=0.5, retries=1, max_workers=1)
    sequential_time = time.time() - start_time
    
    # Тест с max_workers=10 (параллельно)
    start_time = time.time()
    results_parallel = check_upstreams(upstreams, timeout=0.5, retries=1, max_workers=10)
    parallel_time = time.time() - start_time
    
    # Параллельная обработка должна быть быстрее
    assert parallel_time < sequential_time, f"Parallel ({parallel_time:.2f}s) should be faster than sequential ({sequential_time:.2f}s)"
    
    # Результаты должны быть одинаковыми
    assert len(results_sequential) == len(results_parallel)
    for name in upstreams.keys():
        assert len(results_sequential[name]) == len(results_parallel[name])

def test_resolve_parallel_performance(monkeypatch):
    """Тест производительности параллельного резолвинга"""
    import socket
    
    # Мокаем socket.gethostbyname_ex для быстрого теста
    original_gethostbyname_ex = socket.gethostbyname_ex
    
    def mock_gethostbyname_ex(host):
        time.sleep(0.01)  # Имитируем задержку DNS
        return (host, [], ["127.0.0.1"])
    
    monkeypatch.setattr(socket, "gethostbyname_ex", mock_gethostbyname_ex)
    
    upstreams = {}
    for i in range(20):
        upstreams[f"backend{i}"] = [f"example{i}.com:80"]
    
    # Тест с max_workers=1
    start_time = time.time()
    results_sequential = resolve_upstreams(upstreams, max_workers=1)
    sequential_time = time.time() - start_time
    
    # Тест с max_workers=10
    start_time = time.time()
    results_parallel = resolve_upstreams(upstreams, max_workers=10)
    parallel_time = time.time() - start_time
    
    # Параллельная обработка должна быть значительно быстрее
    assert parallel_time < sequential_time * 0.5, f"Parallel ({parallel_time:.2f}s) should be much faster than sequential ({sequential_time:.2f}s)"
    
    # Результаты должны быть одинаковыми
    assert len(results_sequential) == len(results_parallel)

def test_max_workers_scaling():
    """Тест масштабирования с разным количеством потоков"""
    upstreams = {"backend": ["127.0.0.1:99999"] * 10}
    
    times = {}
    for workers in [1, 5, 10, 20]:
        start_time = time.time()
        check_upstreams(upstreams, timeout=0.1, retries=1, max_workers=workers)
        times[workers] = time.time() - start_time
    
    # Больше потоков должно быть быстрее (до определенного предела)
    assert times[10] <= times[1], "10 workers should be faster than 1 worker"
    assert times[20] <= times[1], "20 workers should be faster than 1 worker"




from setuptools import setup, find_packages
from setuptools.command.install import install
from setuptools.command.install_lib import install_lib
import sys
import os


class PostInstallCommand(install):
    """
    Кастомная команда установки для автоматической установки автодополнения.
    """
    def run(self):
        # Выполняем стандартную установку
        install.run(self)
        
        # Устанавливаем автодополнение и создаем конфиг
        self._run_post_install()
    
    def _run_post_install(self):
        """Выполняет пост-установочные действия."""
        try:
            from setup_hooks import post_install
            post_install()
        except Exception as e:
            # Не критично, если не удалось установить completion
            print(f"⚠ Не удалось установить автодополнение автоматически: {e}", file=sys.stderr)
            print("  Выполните вручную: nginx-lens init", file=sys.stderr)


class PostInstallLibCommand(install_lib):
    """
    Альтернативный способ выполнения пост-установочных действий через install_lib.
    Это вызывается даже при wheel установке.
    """
    def run(self):
        # Выполняем стандартную установку библиотек
        install_lib.run(self)
        
        # Выполняем пост-установочные действия
        try:
            # Пробуем импортировать из установленного пакета
            try:
                from utils.init_helpers import post_install
            except ImportError:
                # Fallback на локальный модуль (при первой установке)
                from setup_hooks import post_install
            post_install()
        except Exception as e:
            # Не критично, если не удалось установить completion
            print(f"⚠ Не удалось установить автодополнение автоматически: {e}", file=sys.stderr)
            print("  Выполните вручную: nginx-lens init", file=sys.stderr)


setup(
    name="nginx-lens",
    version="0.5.4",
    description="CLI-инструмент для анализа, визуализации и диагностики конфигураций Nginx",
    author="Daniil Astrouski",
    author_email="shelovesuastra@gmail.com",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "typer[all]>=0.9.0",
        "rich>=13.0.0",
        "requests>=2.25.0",
        "dnspython>=2.0.0",
        "pyyaml>=5.4",
    ],
    extras_require={
        "dev": [
            "pytest>=7.0.0",
            "pytest-cov>=4.0.0",
        ],
    },
    entry_points={
        "console_scripts": [
            "nginx-lens=commands.cli:app",
        ],
    },
    python_requires=">=3.8",
    cmdclass={
        'install': PostInstallCommand,
        'install_lib': PostInstallLibCommand,
    },
) 


"""
Утилиты для отображения прогресса операций.
"""
from typing import Optional, Callable, Any
from rich.progress import Progress, SpinnerColumn, BarColumn, TextColumn, TimeElapsedColumn, TimeRemainingColumn, TaskID
from rich.console import Console
import signal
import sys

console = Console()


class ProgressManager:
    """
    Менеджер для отображения прогресса операций с поддержкой отмены.
    """
    
    def __init__(self, description: str = "Обработка", show_progress: bool = True):
        """
        Инициализация менеджера прогресса.
        
        Args:
            description: Описание операции
            show_progress: Показывать ли прогресс-бар
        """
        self.description = description
        self.show_progress = show_progress
        self.progress: Optional[Progress] = None
        self.task_id: Optional[TaskID] = None
        self.cancelled = False
        
        # Обработка сигнала прерывания
        self._original_sigint = signal.signal(signal.SIGINT, self._handle_interrupt)
    
    def _handle_interrupt(self, signum, frame):
        """Обработка прерывания (Ctrl+C)."""
        self.cancelled = True
        if self.progress:
            self.progress.stop()
        console.print("\n[yellow]Операция отменена пользователем[/yellow]")
        sys.exit(130)  # Стандартный exit code для SIGINT
    
    def __enter__(self):
        """Вход в контекстный менеджер."""
        if self.show_progress:
            self.progress = Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                BarColumn(),
                TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
                TimeElapsedColumn(),
                TimeRemainingColumn(),
                console=console,
                transient=False,
            )
            self.progress.start()
            self.task_id = self.progress.add_task(self.description, total=None)
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        """Выход из контекстного менеджера."""
        if self.progress:
            self.progress.stop()
        # Восстанавливаем оригинальный обработчик сигнала
        signal.signal(signal.SIGINT, self._original_sigint)
        return False
    
    def update(self, completed: int, total: Optional[int] = None, description: Optional[str] = None):
        """
        Обновляет прогресс.
        
        Args:
            completed: Количество завершенных задач
            total: Общее количество задач (если None, используется спиннер)
            description: Описание текущей операции
        """
        if self.progress and self.task_id is not None:
            if total is not None:
                self.progress.update(self.task_id, total=total, completed=completed)
            if description:
                self.progress.update(self.task_id, description=description)
    
    def advance(self, advance: int = 1):
        """
        Увеличивает прогресс на указанное значение.
        
        Args:
            advance: На сколько увеличить прогресс
        """
        if self.progress and self.task_id is not None:
            self.progress.advance(self.task_id, advance)


def with_progress(
    description: str,
    total: Optional[int] = None,
    show_progress: bool = True
) -> Callable:
    """
    Декоратор для добавления прогресс-бара к функции.
    
    Args:
        description: Описание операции
        total: Общее количество итераций (если известно)
        show_progress: Показывать ли прогресс-бар
        
    Returns:
        Декорированная функция
    """
    def decorator(func: Callable) -> Callable:
        def wrapper(*args, **kwargs):
            with ProgressManager(description=description, show_progress=show_progress) as pm:
                # Передаем менеджер прогресса в функцию
                kwargs['progress_manager'] = pm
                if total is not None:
                    pm.update(0, total=total)
                return func(*args, **kwargs)
        return wrapper
    return decorator




"""
Вспомогательные функции для инициализации nginx-lens.
Используются как при установке пакета, так и при выполнении команды init.
"""
import os
import sys
from pathlib import Path
from typing import Optional, Tuple

try:
    import yaml
except ImportError:
    yaml = None


def detect_shell() -> Optional[str]:
    """
    Определяет текущий shell пользователя.
    
    Returns:
        Имя shell (bash, zsh, fish, powershell) или None
    """
    shell = os.environ.get('SHELL', '')
    
    if 'bash' in shell.lower():
        return 'bash'
    elif 'zsh' in shell.lower():
        return 'zsh'
    elif 'fish' in shell.lower():
        return 'fish'
    elif 'powershell' in shell.lower() or 'pwsh' in shell.lower():
        return 'powershell'
    
    # Дополнительная проверка через ps
    try:
        import subprocess
        result = subprocess.run(['ps', '-p', str(os.getppid()), '-o', 'comm='], 
                              capture_output=True, text=True, timeout=2)
        if result.returncode == 0:
            shell_name = result.stdout.strip()
            if 'bash' in shell_name:
                return 'bash'
            elif 'zsh' in shell_name:
                return 'zsh'
            elif 'fish' in shell_name:
                return 'fish'
    except Exception:
        pass
    
    return None


def get_completion_script(shell: str) -> str:
    """
    Генерирует completion скрипт для указанного shell.
    
    Args:
        shell: Тип shell (bash, zsh, fish, powershell)
        
    Returns:
        Строка с completion скриптом
    """
    commands = [
        "health", "analyze", "tree", "diff", "route", "include",
        "graph", "logs", "syntax", "resolve", "validate", "metrics", "init"
    ]
    
    if shell == "bash":
        script = f"""# Bash completion for nginx-lens
_nginx_lens_completion() {{
    local cur="${{COMP_WORDS[COMP_CWORD]}}"
    local commands="{' '.join(commands)}"
    
    if [[ ${{COMP_CWORD}} -eq 1 ]]; then
        COMPREPLY=($(compgen -W "$commands" -- "$cur"))
    else
        COMPREPLY=($(compgen -f -- "$cur"))
    fi
}}

complete -F _nginx_lens_completion nginx-lens
"""
    elif shell == "zsh":
        script = f"""# Zsh completion for nginx-lens
#compdef nginx-lens

_nginx_lens() {{
    local -a commands
    commands=(
        {' '.join(f'"{cmd}":{cmd}' for cmd in commands)}
    )
    
    _describe 'command' commands
}}

_nginx_lens "$@"
"""
    elif shell == "fish":
        script = f"""# Fish completion for nginx-lens
complete -c nginx-lens -f -a "{' '.join(commands)}"
"""
    elif shell == "powershell":
        script = f"""# PowerShell completion for nginx-lens
Register-ArgumentCompleter -Native -CommandName nginx-lens -ScriptBlock {{
    param($commandName, $wordToComplete, $cursorPosition)
    
    $commands = @({', '.join(f'"{cmd}"' for cmd in commands)})
    
    $commands | Where-Object {{ $_ -like "$wordToComplete*" }} | ForEach-Object {{
        [System.Management.Automation.CompletionResult]::new($_, $_, 'ParameterValue', $_)
    }}
}}
"""
    else:
        return ""
    
    return script


def install_completion(shell: str, dry_run: bool = False) -> bool:
    """
    Устанавливает completion скрипт для указанного shell.
    
    Args:
        shell: Тип shell (bash, zsh, fish, powershell)
        dry_run: Если True, только проверяет возможность установки
        
    Returns:
        True если установка успешна, False иначе
    """
    script = get_completion_script(shell)
    if not script:
        return False
    
    home = Path.home()
    
    try:
        if shell == "bash":
            # Устанавливаем в ~/.bashrc
            bashrc = home / ".bashrc"
            if dry_run:
                return bashrc.exists() or (home / ".bash_profile").exists()
            
            # Проверяем, не установлено ли уже
            if bashrc.exists():
                with open(bashrc, 'r') as f:
                    if 'nginx-lens' in f.read():
                        return True  # Уже установлено
            
            # Добавляем в .bashrc
            with open(bashrc, 'a') as f:
                f.write(f"\n# nginx-lens completion\n{script}\n")
            return True
            
        elif shell == "zsh":
            # Устанавливаем в ~/.zshrc
            zshrc = home / ".zshrc"
            if dry_run:
                return zshrc.exists()
            
            # Проверяем, не установлено ли уже
            if zshrc.exists():
                with open(zshrc, 'r') as f:
                    if 'nginx-lens' in f.read():
                        return True  # Уже установлено
            
            # Добавляем в .zshrc
            with open(zshrc, 'a') as f:
                f.write(f"\n# nginx-lens completion\n{script}\n")
            return True
            
        elif shell == "fish":
            # Устанавливаем в ~/.config/fish/completions/nginx-lens.fish
            completions_dir = home / ".config" / "fish" / "completions"
            completion_file = completions_dir / "nginx-lens.fish"
            
            if dry_run:
                return completions_dir.exists() or completions_dir.parent.exists()
            
            # Создаем директорию если нужно
            completions_dir.mkdir(parents=True, exist_ok=True)
            
            # Записываем completion файл
            with open(completion_file, 'w') as f:
                f.write(script)
            return True
            
        elif shell == "powershell":
            # Устанавливаем в профиль PowerShell
            try:
                import subprocess
                result = subprocess.run(
                    ['powershell', '-Command', '$PROFILE'],
                    capture_output=True,
                    text=True,
                    timeout=5
                )
                if result.returncode == 0:
                    profile_path = Path(result.stdout.strip())
                    if dry_run:
                        return profile_path.parent.exists()
                    
                    # Создаем директорию профиля если нужно
                    profile_path.parent.mkdir(parents=True, exist_ok=True)
                    
                    # Проверяем, не установлено ли уже
                    if profile_path.exists():
                        with open(profile_path, 'r') as f:
                            if 'nginx-lens' in f.read():
                                return True  # Уже установлено
                    
                    # Добавляем в профиль
                    with open(profile_path, 'a') as f:
                        f.write(f"\n# nginx-lens completion\n{script}\n")
                    return True
            except Exception:
                pass
            
            return False
    except Exception:
        return False
    
    return False


def create_default_config() -> Tuple[bool, Optional[str]]:
    """
    Создает директорию /opt/nginx-lens и дефолтный конфиг.
    
    Returns:
        Tuple[bool, Optional[str]]: (успешно ли, сообщение об ошибке или None)
    """
    if yaml is None:
        return False, "PyYAML не установлен"
    
    opt_dir = Path("/opt/nginx-lens")
    config_file = opt_dir / "config.yaml"
    
    # Дефолтный конфиг
    default_config = {
        "defaults": {
            "timeout": 2.0,
            "retries": 1,
            "mode": "tcp",
            "max_workers": 10,
            "dns_cache_ttl": 300,
            "top": 10,
        },
        "output": {
            "colors": True,
            "format": "table",
        },
        "cache": {
            "enabled": True,
            "ttl": 300,
        },
        "validate": {
            "check_syntax": True,
            "check_analysis": True,
            "check_upstream": True,
            "check_dns": False,
            "nginx_path": "nginx",
        }
    }
    
    try:
        # Создаем директорию если нужно
        if not opt_dir.exists():
            try:
                opt_dir.mkdir(parents=True, mode=0o755, exist_ok=True)
            except PermissionError as e:
                return False, f"Нет прав для создания директории /opt/nginx-lens (требуются права root): {e}"
            except OSError as e:
                return False, f"Ошибка при создании директории /opt/nginx-lens: {e}"
        
        # Проверяем, существует ли уже конфиг
        if config_file.exists():
            # Конфиг уже существует, не перезаписываем
            return True, None
        
        # Создаем дефолтный конфиг
        try:
            with open(config_file, 'w') as f:
                yaml.dump(default_config, f, default_flow_style=False, allow_unicode=True, sort_keys=False)
        except PermissionError as e:
            return False, f"Нет прав для создания файла {config_file} (требуются права root): {e}"
        except IOError as e:
            return False, f"Ошибка записи в файл {config_file}: {e}"
        
        # Устанавливаем права доступа
        try:
            config_file.chmod(0o644)
        except Exception:
            # Не критично, если не удалось установить права
            pass
        
        return True, None
    except Exception as e:
        return False, f"Неожиданная ошибка: {e}"


def post_install():
    """
    Выполняется после установки пакета для автоматической установки completion и создания конфига.
    """
    # Создаем дефолтный конфиг в /opt/nginx-lens
    try:
        success, error_msg = create_default_config()
        if success:
            print("✓ Конфигурационный файл создан в /opt/nginx-lens/config.yaml", file=sys.stderr)
        else:
            print(f"⚠ Не удалось создать конфиг в /opt/nginx-lens: {error_msg}", file=sys.stderr)
            print("  Для создания конфига выполните:", file=sys.stderr)
            print("    sudo mkdir -p /opt/nginx-lens", file=sys.stderr)
            print("    sudo nginx-lens init", file=sys.stderr)
            print("  Или создайте конфиг вручную в ~/.nginx-lens/config.yaml", file=sys.stderr)
    except Exception as e:
        print(f"⚠ Ошибка при создании конфига: {e}", file=sys.stderr)
    
    # Определяем shell
    shell = detect_shell()
    
    if not shell:
        # Не можем определить shell, пропускаем установку
        return
    
    # Пробуем установить completion
    try:
        if install_completion(shell, dry_run=False):
            print(f"✓ Автодополнение для {shell} установлено автоматически", file=sys.stderr)
            if shell in ['bash', 'zsh']:
                print(f"  Выполните 'source ~/.{shell}rc' для применения изменений", file=sys.stderr)
        else:
            print(f"⚠ Не удалось автоматически установить автодополнение для {shell}", file=sys.stderr)
            print(f"  Выполните вручную: nginx-lens completion install {shell}", file=sys.stderr)
    except Exception as e:
        print(f"⚠ Ошибка при установке автодополнения: {e}", file=sys.stderr)
        print(f"  Выполните вручную: nginx-lens completion install {shell}", file=sys.stderr)




"""
Команда для инициализации nginx-lens (создание конфига и установка completion).
"""
import sys
import typer
from rich.console import Console
from rich.panel import Panel
from rich import box

app = typer.Typer(help="Инициализация nginx-lens: создание конфига и установка автодополнения.")
console = Console()


@app.command()
def init(
    force: bool = typer.Option(False, "--force", "-f", help="Перезаписать существующий конфиг")
):
    """
    Инициализирует nginx-lens: создает конфиг и устанавливает автодополнение.
    
    Эта команда выполняет те же действия, что должны выполняться автоматически
    при установке пакета через pip. Если автоматическая установка не сработала,
    выполните эту команду вручную.
    
    Примеры:
        nginx-lens init
        nginx-lens init --force
    """
    try:
        from utils.init_helpers import post_install, create_default_config, detect_shell, install_completion
        
        # Создаем конфиг
        console.print("[cyan]Создание конфигурационного файла...[/cyan]")
        success, error_msg = create_default_config()
        if success:
            console.print("[green]✓ Конфигурационный файл создан в /opt/nginx-lens/config.yaml[/green]")
        else:
            if "Permission denied" in error_msg or "Нет прав" in error_msg:
                console.print(f"[yellow]⚠ Не удалось создать конфиг в /opt/nginx-lens: {error_msg}[/yellow]")
                console.print("[yellow]  Попробуйте выполнить с правами root:[/yellow]")
                console.print("[yellow]    sudo nginx-lens init[/yellow]")
                console.print("[yellow]  Или создайте конфиг вручную в ~/.nginx-lens/config.yaml[/yellow]")
            else:
                console.print(f"[red]✗ Ошибка при создании конфига: {error_msg}[/red]")
        
        # Устанавливаем completion
        console.print("\n[cyan]Установка автодополнения...[/cyan]")
        shell = detect_shell()
        if not shell:
            console.print("[yellow]⚠ Не удалось определить shell. Пропускаем установку автодополнения.[/yellow]")
            console.print("[yellow]  Выполните вручную: nginx-lens completion install <shell>[/yellow]")
        else:
            try:
                if install_completion(shell, dry_run=False):
                    console.print(f"[green]✓ Автодополнение для {shell} установлено[/green]")
                    if shell in ['bash', 'zsh']:
                        console.print(f"[yellow]  Выполните 'source ~/.{shell}rc' для применения изменений[/yellow]")
                else:
                    console.print(f"[yellow]⚠ Не удалось установить автодополнение для {shell}[/yellow]")
                    console.print(f"[yellow]  Выполните вручную: nginx-lens completion install {shell}[/yellow]")
            except Exception as e:
                console.print(f"[red]✗ Ошибка при установке автодополнения: {e}[/red]")
                console.print(f"[yellow]  Выполните вручную: nginx-lens completion install {shell}[/yellow]")
        
        console.print("\n[green]✓ Инициализация завершена![/green]")
        
    except ImportError as e:
        console.print(f"[red]✗ Ошибка импорта: {e}[/red]")
        console.print("[yellow]  Убедитесь, что nginx-lens установлен корректно.[/yellow]")
        sys.exit(1)
    except Exception as e:
        console.print(f"[red]✗ Неожиданная ошибка: {e}[/red]")
        sys.exit(1)

